/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalEnvelope.h"
#include "HvSignalVar.h"
#include "HvControlIf.h"
#include "HvMath.h"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvControlCast.h"
#include "HvControlBinop.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_Bgso6eib_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6fVQmUxU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_EFBebiGO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_NKBcZAse_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_toIOgbRe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_lzgo6i8K_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Wq7CyHer_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_7Uiy1LvH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PLpi36a6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zGCnRlEF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_lqOtCuox_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_KEUqsLrd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_p3f6o6AK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_VWrAzAP2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_IGoAzb4Q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_5jETjNXo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_R5TlFV6B_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_SFH3BdIf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_EkQEIHkz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rjVWUxYU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gEtFpv5k_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Tpn0i9jl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ueJhOIhJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_cDb4Kpow_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_D3sCgxpp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_cblAHz0g_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_YSoQ2nvB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Qm2vBQdf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Zcho4fSM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_tuzuRLEF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_hhqkMOSe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_i4LQJhOI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_EtKxQqwP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_CMQEe0ss_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_rZfKs4wA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_1eK7bwAd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ncOm83eM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Ne3ggvFN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xMH2oIs7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_7Tl7zt1I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_62E1EOrG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_39eaYQIW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_SFjxKift_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_FklKCWYM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_JuEAs6zr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_zs2V6Td5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_FruWfrXq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ffOlNK8H_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_1tgsfwKf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_k7o94CJW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_P3oCaiig_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_ycPxWlIZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_VKdya0p8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_NDsCZ31x_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_G76v7hqf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Dh18kYc8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_T9lkeD4Z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_iaXvVf0p_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_NeXSAmmH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ecEqd28S_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_BSz2TdFV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_JlcTcKJF_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_39vfcpei;
  SignalEnvelope sEnv_SFH3BdIf;
  SignalEnvelope sEnv_EkQEIHkz;
  SignalEnvelope sEnv_rZfKs4wA;
  SignalEnvelope sEnv_1eK7bwAd;
  SignalLine sLine_cL7OCflh;
  SignalLine sLine_Id2ImJFk;
  ControlBinop cBinop_Bgso6eib;
  ControlBinop cBinop_6fVQmUxU;
  ControlBinop cBinop_EFBebiGO;
  ControlBinop cBinop_NKBcZAse;
  ControlBinop cBinop_Wq7CyHer;
  ControlBinop cBinop_PLpi36a6;
  ControlBinop cBinop_zGCnRlEF;
  ControlBinop cBinop_lqOtCuox;
  ControlVar cVar_KEUqsLrd;
  ControlVar cVar_VWrAzAP2;
  SignalVarf sVarf_zLG9uswr;
  ControlVar cVar_IGoAzb4Q;
  ControlIf cIf_5jETjNXo;
  ControlIf cIf_R5TlFV6B;
  ControlBinop cBinop_rjVWUxYU;
  ControlBinop cBinop_Tpn0i9jl;
  ControlBinop cBinop_ueJhOIhJ;
  ControlBinop cBinop_cblAHz0g;
  ControlBinop cBinop_Qm2vBQdf;
  SignalVarf sVarf_GUJY0q5g;
  ControlVar cVar_i4LQJhOI;
  ControlIf cIf_EtKxQqwP;
  ControlIf cIf_CMQEe0ss;
  ControlBinop cBinop_ncOm83eM;
  ControlBinop cBinop_xMH2oIs7;
  ControlBinop cBinop_7Tl7zt1I;
  ControlBinop cBinop_SFjxKift;
  ControlBinop cBinop_JuEAs6zr;
  ControlVar cVar_1tgsfwKf;
  ControlBinop cBinop_k7o94CJW;
  SignalVarf sVarf_uQNu1JYO;
  SignalVarf sVarf_vlf1FTBB;
  SignalVarf sVarf_FFyu1gIt;
  SignalVarf sVarf_DzRq7MbN;
  ControlBinop cBinop_NDsCZ31x;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
